//---------------------------------------------------------------------------
//      DS2432, DS1961S
//---------------------------------------------------------------------------
#pragma hdrstop
#include "DS2432.h"
#include "ProgressBar.h"        //   -
#include "MainProgram.h"        //    
#include "ChipFunctions.h"      //      
#include "PortFunctions.h"      //    
#include "CalcCRC.h"            //   CRC
#include "SendMsgFunctions.h"   //     
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0x0F) (  1, 2, 3, 4) -------------------
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2, Input.Size, Input.AreaNumber
// output:      Test - ,      (0xFF) + Success
DS2432_OUTPUT_DATA DS2432_WriteToScratchpad(DS2432_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned short int CRC16;             //   CRC16
  DS2432_OUTPUT_DATA OutputData;        //   
  unsigned int Address;           		//   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //   
  Address=InputData.TA1+0x100*InputData.TA2;             //    ,    

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Write Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x0F;                                  // Write Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //     CRC16
  CRC16=0; CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);      //   CRC16   Write Scratchpad
  Sleep(10);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);               //   CRC16
  Sleep(1);
  // TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);               //   CRC16
  Sleep(1);
  //***     ***
  for(unsigned int i=0; i<InputData.Size; i++)          //   ,   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    switch(InputData.AreaNumber)
    { case 1:   wr_buf.Data[1]=SelectedChip.Area1[Address-SelectedChip.Area1Start];   //    
                break;
      case 2:   wr_buf.Data[1]=SelectedChip.Area2[Address-SelectedChip.Area2Start];   //    
                break;
      case 3:   wr_buf.Data[1]=SelectedChip.Area3[Address-SelectedChip.Area3Start];   //    
                break;
      case 4:   wr_buf.Data[1]=SelectedChip.Area4[i];                                 //   
                break;
      default:  return OutputData;
    }
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);             //   CRC16
    Address+=1;                                         //  
    LBar->ProgressBar1->Position=100*(i+1)/InputData.Size;      //    -
    Sleep(1);
  }
  //  CRC16 (2 , )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  //   CRC16
  if(CRC16)     SendMsgToLogForm(16,"");          //  CRC16  //    -     
  // ,     (  ,    1-)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;          //    Test   
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0xAA) (  Area4) --------------------------
//---------------------------------------------------------------------------
// output:      Test - ,      (0xFF), TA1, TA2, ES, Success
DS2432_OUTPUT_DATA DS2432_ReadFromScratchpad()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS2432_OUTPUT_DATA OutputData;        //   
  unsigned short int CRC16;             //   CRC16

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Scratchpad-
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xAA;                                  // Read Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=0; CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);      //     CRC16
  Sleep(10);
  //*** READ TA1, TA2, E/S ***
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                          // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();          //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA1
  rd_buf=ReadFrom();                            // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();              //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.TA1=rd_buf.Data[0];                //  TA1
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);       //   CRC16
  Sleep(1);
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                          // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();          //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA2
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.TA2=rd_buf.Data[0];                        //  TA2
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  ES
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.ES=rd_buf.Data[0];                         //  ES
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  //***    ***
  for(unsigned int i=0; i<SelectedChip.Area4Size; i++)  //     (Area4)
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                              // 
    if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    SelectedChip.Area4[i]=rd_buf.Data[0];                               //      
    CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);                             //   CRC16
    LBar->ProgressBar1->Position=100*(i+1)/SelectedChip.Area4Size;      //    -
    Sleep(1);
  }
  //  CRC16 (2 , )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  //   CRC16
  if(CRC16)     SendMsgToLogForm(16,"");          //  CRC16  //    -     
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---    (0x5A) (     ) ---
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2, Input.ES
// output:      Test - ,      (0xFF) + Success
DS2432_OUTPUT_DATA DS2432_LoadFirstSecret(DS2432_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS2432_OUTPUT_DATA OutputData;        //   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //   

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Load First Secret
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x5A;                                  // Load First Secret
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  // TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  // ES
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.ES;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  // ,     (  ,    1-)
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;          //    Test   
}
//---------------------------------------------------------------------------
//---    (0xF0) (  Area1, Area2, Area3) ---------------
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2, Input.Size, Input.AreaNumber
// output:      Test - ,      (0xAA) + Success
DS2432_OUTPUT_DATA DS2432_ReadFromMemory(DS2432_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS2432_OUTPUT_DATA OutputData;        //   
  unsigned int Address;           		//   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //  
  Address=InputData.TA1+0x100*InputData.TA2;             //    ,    

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //   EEPROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xF0;                                  // Read EEPROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //***    ***
  for(unsigned int i=0; i<InputData.Size; i++)          //   
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                                  // 
    if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    switch(InputData.AreaNumber)
    { case 1:   SelectedChip.Area1[Address-SelectedChip.Area1Start]=rd_buf.Data[0];     //     Area1
                break;
      case 2:   SelectedChip.Area2[Address-SelectedChip.Area2Start]=rd_buf.Data[0];     //     Area2
                break;
      case 3:   SelectedChip.Area3[Address-SelectedChip.Area3Start]=rd_buf.Data[0];     //     Area3
                break;
      default:  break;
    }
    Sleep(1);
    Address+=1;                                                 //  

    LBar->ProgressBar1->Position=100*(i+1)/InputData.Size;      //    -
    Sleep(1);
  }
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                    // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD  EEPROM (0x55) -----------------------------
//--- (    R/W ,  ) ---------------
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2, Input.ES
// output:      Test - ,      (0xAA) + Success
DS2432_OUTPUT_DATA DS2432_CopyScratchpadToMemory(DS2432_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS2432_OUTPUT_DATA OutputData;        //   
  MAC_DATA CalculatedMac;               //    MAC

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=0x55;                            // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Copy Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Copy Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  E/S
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.ES;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //  MAC
  CalculatedMac=CalculateMAC(InputData.TA1, InputData.TA2);
  //   MAC
  //  4  E
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[0];                  //    E
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[0]>>8;               //    E
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[0]>>16;              //    E
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[0]>>24;              //    E
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  4  D
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[1];                  //    D
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[1]>>8;               //    D
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[1]>>16;              //    D
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[1]>>24;              //    D
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  4  C
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[2];                  //    C
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[2]>>8;               //    C
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[2]>>16;              //    C
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[2]>>24;              //    C
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  4  B
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[3];                  //    B
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[3]>>8;               //    B
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[3]>>16;              //    B
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[3]>>24;              //    B
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  4  A
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[4];                  //    A
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[4]>>8;                  //    A
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[4]>>16;                  //    A
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=CalculatedMac.MAC[4]>>24;                  //    A
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(20);
  // ,    
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //    - ,  MAC 
  if(OutputData.Test==0)        SendMsgToLogForm(31,"");        //  MAC
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---  SCRATCHPAD (0xA3) (   1) -------------------
//---------------------------------------------------------------------------
//   8       , 
//      (   ).  
//     LoadFirstSecret,    
//       (    )
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2
// output:      Test - ,      (0xFF) + Success
DS2432_OUTPUT_DATA DS2432_RefreshScratchpad(DS2432_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned short int CRC16;             //   CRC16
  DS2432_OUTPUT_DATA OutputData;        //   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //   

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Refresh Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xA3;                                  // Refresh Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //     CRC16
  CRC16=0; CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);      //   CRC16   Refresh Scratchpad
  Sleep(10);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);               //   CRC16
  Sleep(1);
  // TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);               //   CRC16
  Sleep(1);
  //***  8   ***
  // (     ,   ,   ,   )
  for(unsigned int i=0; i<SelectedChip.Area4Size; i++)  //    =  
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=0xFF;                                // next fictive byte
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");     //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);             //   CRC16
    LBar->ProgressBar1->Position=100*(i+1)/InputData.Size;      //    -
    Sleep(1);
  }
  //  CRC16 (2 , )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  //   CRC16
  if(CRC16)     SendMsgToLogForm(16,"");          //  CRC16  //    -     
  // ,     (  ,    1-)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;          //    Test   
}
//---------------------------------------------------------------------------

 