//---------------------------------------------------------------------------
//      MAX31826
//---------------------------------------------------------------------------
#pragma hdrstop
#include "MAX31826.h"
#include "ProgressBar.h"        //   -
#include "MainProgram.h"        //    
#include "ChipFunctions.h"      //      
#include "PortFunctions.h"      //    
#include "CalcCRC.h"            //   CRC
#include "SendMsgFunctions.h"   //     
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---   SCRATCHPAD 2 (0x0F) (  1, 2) -----------------------
//---------------------------------------------------------------------------
// input:       Input.SA, Input.Size, Input.AreaNumber
// output:      Success
MAX31826_OUTPUT_DATA MAX31826_WriteToScratchpad(MAX31826_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned char CRC8;                   //   CRC8
  MAX31826_OUTPUT_DATA OutputData;      //   
  unsigned int Address;           	//   

  OutputData.Test=0;
  OutputData.Success=false;             //   
  Address=InputData.SA;                 //    ,    

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=0x55;                            // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                 //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                          // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];           // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Write Scratchpad2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x0F;                                  // Write Scratchpad2
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //     CRC8
  CRC8=0;
  for(int j=0; j<8; j++)
  {     CRC8=CalcCRC8(CRC8, wr_buf.Data[1]);      //   CRC8   Write Scratchpad
        wr_buf.Data[1]=wr_buf.Data[1]>>1;         //   (   )
  }
  Sleep(10);
  //  SA
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=InputData.SA;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  for(int j=0; j<8; j++)
  {     CRC8=CalcCRC8(CRC8, wr_buf.Data[1]);      //   CRC8
        wr_buf.Data[1]=wr_buf.Data[1]>>1;         //   (   )
  }
  Sleep(1);
  //***     ***
  for(unsigned int i=0; i<InputData.Size; i++)    //   ,   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                          // SendByte_1W
    switch(InputData.AreaNumber)
    { case 1:   wr_buf.Data[1]=SelectedChip.Area1[Address-SelectedChip.Area1Start];   //    
                break;
      case 2:   wr_buf.Data[1]=SelectedChip.Area2[Address-SelectedChip.Area2Start];   //     2
                break;
      default:  return OutputData;
    }
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");     //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    for(int j=0; j<8; j++)
    {     CRC8=CalcCRC8(CRC8, wr_buf.Data[1]);      //   CRC8
          wr_buf.Data[1]=wr_buf.Data[1]>>1;         //   (   )
    }
    Address+=1;                                         //  
    LBar->ProgressBar1->Position=100*(i+1)/InputData.Size;      //    -
    Sleep(1);
  }
  //  CRC8 (1 )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   ( CRC8)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  for(int j=0; j<8; j++)
  {     CRC8=CalcCRC8(CRC8, rd_buf.Data[0]);      //   CRC8
        rd_buf.Data[0]=rd_buf.Data[0]>>1;         //   (   )
  }
  Sleep(1);
  //   CRC8
  if(CRC8)     SendMsgToLogForm(14,"");           //  CRC8  //    -     
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                   //    -
  OutputData.Success=true;                        //   
  return OutputData;
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0xAA) (  Area2  Area3) ----------------
//---------------------------------------------------------------------------
// input:       MAX31826_INPUT_DATA.AreaNumber
// output:      Success
MAX31826_OUTPUT_DATA MAX31826_ReadFromScratchpad(MAX31826_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;      //   
  unsigned char CRC8;                   //   CRC8

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Scratchpad-
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  if(InputData.AreaNumber == 2) wr_buf.Data[1]=0xAA;                                  // Read Scratchpad 2
  else                          wr_buf.Data[1]=0xBE;                                  // Read Scratchpad 1
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //     CRC8
  CRC8=0;
  for(int j=0; j<8; j++)
  {     CRC8=CalcCRC8(CRC8, wr_buf.Data[1]);      //   CRC8   Write Scratchpad
        wr_buf.Data[1]=wr_buf.Data[1]>>1;         //   (   )
  }
  Sleep(10);
  if(InputData.AreaNumber == 2)
  {     //***    ***
        wr_buf.Size=2;
        wr_buf.Data[0]=0x61;                                  // SendByte_1W
        wr_buf.Data[1]=InputData.SA;                          // Start Address
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                                 LBar->Free();            //    -
                                 ClosePort();
                                 return OutputData;
                               }
        for(int j=0; j<8; j++)
        {       CRC8=CalcCRC8(CRC8, wr_buf.Data[1]);      //   CRC8   
                wr_buf.Data[1]=wr_buf.Data[1]>>1;         //   (   )
        }
        Sleep(1);
  }
  //***    ***
  for(unsigned int i=0; i<8; i++)  //     (Area2,3)
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                              // 
    if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    if(InputData.AreaNumber == 2)       SelectedChip.Area2[i]=rd_buf.Data[0];   //       2
    else                                SelectedChip.Area3[i]=rd_buf.Data[0];   //       3
    for(int j=0; j<8; j++)
    {     CRC8=CalcCRC8(CRC8, rd_buf.Data[0]);      //   CRC8   Read Scratchpad 2
          rd_buf.Data[0]=rd_buf.Data[0]>>1;         //   (   )
    }
    LBar->ProgressBar1->Position=100*(i+1)/8;      //    -
    Sleep(1);
  }
  //  CRC8 (1 )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   ( CRC8)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  for(int j=0; j<8; j++)
  {     CRC8=CalcCRC8(CRC8, rd_buf.Data[0]);      //   CRC8
        rd_buf.Data[0]=rd_buf.Data[0]>>1;         //   (   )
  }
  Sleep(1);
  //   CRC8
  if(CRC8)     SendMsgToLogForm(14,"");           //  CRC8  //    -     
  else         SendMsgToLogForm(13,"");           // CRC8 Ok
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD 2  EEPROM (0x55) ---------------------------
//---------------------------------------------------------------------------
// output:      Success
MAX31826_OUTPUT_DATA MAX31826_CopyScratchpadToMemory()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;      //   

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Copy Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Copy Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  if(Form1->CheckBox1->Checked == false) //      (    )
  {     //  Token (0xA5)
        wr_buf.Size=2;
        wr_buf.Data[0]=0x61;                                  // SendByte_1W
        wr_buf.Data[1]=0xA5;
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
        Sleep(10);
  }
  else                                  //    
  {     //  Token (0xA5) ,       
        wr_buf.Size=2;
        wr_buf.Data[0]=0x67;                                  // SendByteSTRPU_1W
        wr_buf.Data[1]=0xA5;
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
        Sleep(10);
        wr_buf.Size=1;                                        //   
        wr_buf.Data[0]=0x68;                                  // Clear_STRPU_1W
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
  }
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---    (0xF0) (  Area1) -----------------------------
//---------------------------------------------------------------------------
// input:       Input.SA, Input.Size
// output:      Success
MAX31826_OUTPUT_DATA MAX31826_ReadFromMemory(MAX31826_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;      //   
  unsigned int Address;                 //   

  OutputData.Test=0;
  OutputData.Success=false;             //  
  Address=InputData.SA;                 //    ,    

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //   EEPROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xF0;                                  // Read EEPROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //  Address
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.SA;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //***    ***
  for(unsigned int i=0; i<InputData.Size; i++)          //   
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                                  // 
    if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    SelectedChip.Area1[Address-SelectedChip.Area1Start]=rd_buf.Data[0];     //     Area1
    Sleep(1);
    Address+=1;                                                 //  

    LBar->ProgressBar1->Position=100*(i+1)/InputData.Size;      //    -
    Sleep(1);
  }
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---    POWER (0xB4) -----------------------------------
//---------------------------------------------------------------------------
// output:      Success
MAX31826_OUTPUT_DATA MAX31826_ReadPowerSupply()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0x55;                 // 0x0 - parasite power, 0xFF - separate power
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Read Power Supply
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xB4;                                  // Read Power Supply
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];               //   
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  //          
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---   (0x44) ------------------------------------------
//---------------------------------------------------------------------------
// output:      Status + Success
MAX31826_OUTPUT_DATA MAX31826_ConvertT()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  if(Form1->CheckBox1->Checked == false) //      (    )
  {     //  Convert Temperature
        wr_buf.Size=2;
        wr_buf.Data[0]=0x61;                                  // SendByte_1W
        wr_buf.Data[1]=0x44;                                  // Convert T
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
        Sleep(10);
        //         (Test)
        Sleep(1);
        wr_buf.Size=1;
        wr_buf.Data[0]=0x62;                                  //   
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
        Sleep(1);
        //     (Test)
        rd_buf=ReadFrom();                                // 
        if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                           LBar->Free();                      //    -
                           ClosePort();
                           return OutputData;
                         }
        OutputData.Test=rd_buf.Data[0];
        Sleep(1);
  }
  else                                  //    
  {     //  Convert Temperature       
        wr_buf.Size=2;
        wr_buf.Data[0]=0x67;                                  // SendByteSTRPU_1W
        wr_buf.Data[1]=0x44;                                  // Convert T
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
        Sleep(10);
        wr_buf.Size=1;                                        //   
        wr_buf.Data[0]=0x68;                                  // Clear_STRPU_1W
        if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                               LBar->Free();            //    -
                               ClosePort();
                               return OutputData;
                             }
        OutputData.Test=0xFF;   // ,   !
  }
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---    ---------------------------------------
//---------------------------------------------------------------------------
float MAX31826_DecodeTemperature()
{ float OutputValue, Temp1;     //    +   (   )
  unsigned int Temp2;           //    +   (   )

  //     OutputValue
  OutputValue=0; Temp1=0.0625;                          //   0, Temp1 = 2^-4
  Temp2=SelectedChip.Area3[1];                          //   MSB
  Temp2=(Temp2<<8)+SelectedChip.Area3[0];               //      LSB
  //     ( )    
  for(unsigned int i=0; i<11; i++)
  { OutputValue+=Temp1*(Temp2&1);
    Temp2>>=1;  //    
    Temp1*=2;   //      (  )
  }
  //   ( Temp2        )
  if(Temp2)     OutputValue*=-1.0;      //      ,   
  return OutputValue;
}
//---------------------------------------------------------------------------
//---       ----------------------------
//---------------------------------------------------------------------------
// input:       Input.SA -   ,      (0x80 - Low, 0x81 - High)
// output:      Success
MAX31826_OUTPUT_DATA MAX31826_LockMemory(MAX31826_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;      //   

  OutputData.Success=false;             //   

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=0x55;                            // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                 //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                          // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];           // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Write Scratchpad2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x0F;                                  // Write Scratchpad2
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //  SA (0x80  0x81)
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=InputData.SA;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //  55h
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=0x55;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=0x55;                            // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                 //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                          // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];           // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Copy Scratchpad2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Copy Scratchpad2
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");       //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //   A5h
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                            // SendByte_1W
  wr_buf.Data[1]=0xA5;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,""); //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(30);                                      //    /  = 25 
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                   //    -
  OutputData.Success=true;                        //   
  return OutputData;
}
//---------------------------------------------------------------------------
//---   LOCK  --------------------------------------------
//---------------------------------------------------------------------------
// input:       Input.SA: 0x80 -    , 0x81 -    
// output:      Success, Test: Test   
MAX31826_OUTPUT_DATA MAX31826_ReadLockStatus(MAX31826_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  MAX31826_OUTPUT_DATA OutputData;      //   

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //   EEPROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xF0;                                  // Read EEPROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //  Address
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.SA;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //***    ***
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");     //   
                         LBar->Free();                //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   
  rd_buf=ReadFrom();                                  // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");         //   
                     LBar->Free();                    //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];                     //     Test
  //  Reset  
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");     //   
                         LBar->Free();                //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                       //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
